/*
 * Allgemein.java
 * eu.gronos.kostenrechner.model.tenordaten (Kostenrechner)
 */
package eu.gronos.kostenrechner.model.tenordaten;

import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.datatype.XMLGregorianCalendar;

import eu.gronos.kostenrechner.controller.files.XjustizContentHandler;
import eu.gronos.kostenrechner.interfaces.UnterContainerKlasse;
import eu.gronos.kostenrechner.view.KostenJPanel;

/**
 * Unterklasse von {@link VerfahrensDatenContainer} für allgemeine,
 * Registerkarten-übergreifende Einstellungen
 *
 * @author Peter Schuster (setrok)
 * @date 28.12.2014
 */
public class Allgemein implements UnterContainerKlasse {
	@XmlAttribute(name = "speicherdatum")
	public XMLGregorianCalendar speicherdatum;

	/**
	 * b92ii speichert als <code>boolean</code>, ob § 92 Abs. 2 ZPO berücksichtigt
	 * werden soll.
	 */
	@XmlAttribute(name = "p92iiZPO")
	public boolean b92ii;

	/**
	 * {@link #vomHundert92ii} speichert als <code>int</code> die Schwelle für § 92
	 * Abs. 2 ZPO, also bis zu welchem Prozentsatz das Unterliegen unwesentlich sein
	 * soll.
	 */
	@XmlAttribute(name = "p92iiZPOvH") // schwelle
	public int vomHundert92ii = 10;// 10%//double0.1

	/**
	 * {@link #aktenzeichen} soll als {@link String} speichern, welches Aktenzeichen
	 * das Verfahren hat. Wird derzeit nur von {@link XjustizContentHandler} genutzt
	 * und ist ansonsten ohne Funktion
	 */
	@XmlAttribute(name = "aktenzeichen")
	public String aktenzeichen;// = "k.A.";

	/**
	 * {@link #streitwert} speichert als <code>double</code> den allgemeinen
	 * Streitwert
	 */
	@XmlAttribute(name = "streitwert")
	public double streitwert;

	/**
	 * {@link #selectedPanel} speichert als <code>int</code>, welches
	 * {@link KostenJPanel} beim Speichern gerade aktiv war.
	 */
	@XmlAttribute(name = "selectedPanel")
	public int selectedPanel = -1;

	/**
	 * @return immer <code>true</code>, weil keinem {@link KostenJPanel} zugeordnet.
	 * 
	 * @see eu.gronos.kostenrechner.interfaces.UnterContainerKlasse#isEmpty()
	 */
	@Override
	public boolean isEmpty() {
		return true;
	}
}